#unittesting for webscraping.py functions
import unittest

import pandas as pd


#url_builders
class url_builders(unittest.TestCase): # test class

    #trying to import package
    import sys
    import os
    import re
    packagepath = os.path.abspath(__file__)
    packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
    if not packagepath in sys.path:
        sys.path.append(packagepath)

    from network import webscraping



    #set up and tear down
    def setUp(self):
        #print("setup")
        self.topic1 = 'cell'
        self.topic2 = 'cell bio'
        self.topic3 = 'not allowed 9'
        self.expected1_quizlet = 'https://quizlet.com/search?query=cell&type=sets&page=1&creator=teacher'
        self.expected1_cram = 'https://www.cram.com/search?query=cell&search_in%5B%5D=title&search_in%5B%5D=body&search_in%5B%5D=subject&search_in%5B%5D=username&image_filter=exclude_imgs&period=any'
        self.expected2_quizlet = 'https://quizlet.com/search?query=cell-bio&type=sets&page=1&creator=teacher'
        self.expected2_cram = 'https://www.cram.com/search?query=cell+bio&search_in%5B%5D=title&search_in%5B%5D=body&search_in%5B%5D=subject&search_in%5B%5D=username&image_filter=exclude_imgs&period=any'

    def tearDown(self):
        #print("teardown")
        del self.topic1
        del self.topic2
        del self.topic3
        del self.expected1_quizlet
        del self.expected1_cram
        del self.expected2_quizlet
        del self.expected2_cram

    #url_builders function
    def test_url_builders(self): # test case
        self.assertEqual(self.webscraping.url_quizlet(self.topic1), self.expected1_quizlet)
        self.assertEqual(self.webscraping.url_quizlet(self.topic2), self.expected2_quizlet)
        self.assertEqual(self.webscraping.url_cram(self.topic1), self.expected1_cram)
        self.assertEqual(self.webscraping.url_cram(self.topic2), self.expected2_cram)
    def error_url_builders(self):
        self.assertRaises(AssertionError, self.webscraping.url_quizlet, self.topic3)
        self.assertRaises(AssertionError, self.webscraping.url_cram, self.topic3)
unittest.main(argv=[''], verbosity=2, exit=False)





#webscraping functions....
class webscraping_funcs(unittest.TestCase): # test class
    import pandas as pd

    #trying to import package
    import sys
    import os
    import re
    packagepath = os.path.abspath(__file__)
    packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
    if not packagepath in sys.path:
        sys.path.append(packagepath)

    from network import webscraping



    #set up and tear down
    def setUp(self):
        #print("setup")
        import pandas as pd
        self.url_quizlet = 'https://quizlet.com/751206399/the-easy-quiz-flash-cards/'
        self.url_cram = 'https://www.cram.com/flashcards/easy-11264076'
        self.expected_quizlet = pd.DataFrame({'questions':['Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball'], 'answers':['Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball', 'Ball']})
        self.expected_cram = pd.DataFrame({'questions':['What', 'No', 'Blue'], 'answers':['Is', 'Say', 'Man']})

    def tearDown(self):
        #print("teardown")
        del self.url_quizlet
        del self.expected_quizlet
        del self.url_cram
        del self.expected_cram

    #url_builders function
    def test_webscraping_quizlet(self): # test case
        #issue here is that the websites change all the time, so must use override command to get specific card set
        self.assertEqual(None, self.pd.testing.assert_frame_equal(self.webscraping.webscrape_quizlet(override=self.url_quizlet), self.expected_quizlet))
    def test_webscraping_cram(self):
        self.assertEqual(None, self.pd.testing.assert_frame_equal(self.webscraping.webscrape_cram(override=self.url_cram), self.expected_cram))
unittest.main(argv=[''], verbosity=2, exit=False)



##Network subpackage demonstration
#Jonah Edmundson

#run this file in the terminal as:
#python network_demonstration.py

import pandas as pd


#################
# webscraping module
import webscraping as ws
################

#url_quizlet
print("url_quizlet")
print("pass a topic as a string, and a quizlet URL is returned")
testing = ws.url_quizlet("cell bio")
print(testing)


#url_cram
print("\n\nurl_cram")
print("same as url_quizlet, but for cram")
testing2 = ws.url_cram("python practice")
print(testing2)


#webscrape_quizlet
print("\n\nwebscape_quizlet")
print("pass a URL and optional set number, returns a dataframe of questions and answers from quizlet")
df = ws.webscrape_quizlet(testing, setNum=5)
print(df)
print('\n\nOVERRIDE URL')
alt = ws.webscrape_quizlet(override = 'https://quizlet.com/751206399/the-easy-quiz-flash-cards/')
print(alt)


#webscrape_cram
print("\n\nwebscape_cram")
print("pass a URL and optional set number, returns a dataframe of questions and answers from quizlet")
df2 = ws.webscrape_cram(testing2, setNum=5)
print(df2)



#dataframe_builder
print("\n\ndataframe_builder")
print("takes in 2 arrays of website objects with text attributes, returns a single pandas dataframe of the text attributes")
print("since it is difficult to get these text-containing web objects, i will not demonstrate it directly here. this function can therefore only be called from within webscrape_quizlet or webscrape_cram. note the proper syntax below:")
print("lst = dataframe_builder(questionlist, answerlist)")


################
# translation module
import translation as tl
###############

#internet_checker
print("\n\ninternet_checker")
print("returns true if an internet connection exists, otherwise returns false")
print("Do we have internet? -->", tl.internet_checker())


#speed_warning
print("\n\nspeed_warning")
print("prints a speed warning for the webscraping functions")
tl.speed_warning()


#translate
print("\n\ntranslate")
print("takes in a 2 column dataframe labelled \'questions\' and \'answers\' and returns the same dataframe with translated text. also takes in the language to translate to")
print("translating to french....")
print(tl.translate(df, 'fr'))



def internet_checker():
	import urllib.request
	try:
		urllib.request.urlopen('http://google.com')
		return True
	except:
		return False



def speed_warning():
	import time
	print("Functions requiring internet connection may take a while. Please do not interact with your device until the process is finished.")
	time.sleep(4)
	print("Starting in:")
	print("3")
	time.sleep(0.5)
	print("2")
	time.sleep(0.5)
	print("1")
	time.sleep(0.5)



def translate(dataframe, language):
	import pandas as pd
	from googletrans import Translator
	translator = Translator()

	for i in range(len(dataframe.index)):
		dataframe.at[i, "questions"] = translator.translate(dataframe.loc[i, "questions"], dest=language).text
		dataframe.at[i, "answers"] = translator.translate(dataframe.loc[i, "answers"], dest=language).text

	return dataframe



import unittest

#importing stuff
import sys
import os
import re
packagepath = os.path.abspath(__file__)
packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
if not packagepath in sys.path:
	sys.path.append(packagepath)

#import all test classes
from unittesting import translation_testing
from unittesting import webscraping_testing
from unittesting import TestQuizCoreOne
from unittesting import TestQuizCoreTwo


#run everything
def my_suite():
    suite = unittest.TestSuite()
    result = unittest.TestResult()

    #here is where you add your tests
    suite.addTest(unittest.makeSuite(translation_testing.translation))
    suite.addTest(unittest.makeSuite(translation_testing.testing_speed_warning))
    suite.addTest(unittest.makeSuite(webscraping_testing.url_builders))
    suite.addTest(unittest.makeSuite(TestQuizCoreTwo.TestMethods))
    suite.addTest(unittest.makeSuite(TestQuizCoreOne.TestMain))
    suite.addTest(unittest.makeSuite(webscraping_testing.webscraping_funcs))

    runner = unittest.TextTestRunner()
    print(runner.run(suite))
my_suite()



import sys
import os
import re
packagepath = os.path.abspath(__file__)
packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
if not packagepath in sys.path:
	sys.path.append(packagepath)

from core import quiz_methods_testcopy as methods
from core import quiz_main_testcopy
import unittest
import pandas as pd
import random as rand

class TestMethods(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        #Generate a question bank of random questions and answers to test quiz functions on
        self.data = [
            ["Central Processing Unit (CPU)", "Brain of the computer that performs instructions defined by software"],
            ["Software", "Set of instructions that tells the hardware what to do. It is what guides the hardware and tells it how to accomplish each task."],
            ["Operating System (OS)", "Software used to control the computer and its peripheral equipment."],
            ["Test Question", "TestQuestionWithOneWordAnswer"]
                ]

        self.QADataFrame = pd.DataFrame(self.data, columns = ["questions", "answers"])


    def setUp(self):
        #Sample random numbers (without replacement), used to randomly assign questions/answers to quiz objects and test functions
        #For example, quizObjectOne assigned random question 1 should be different than quizObjectTwo asisgned random question 2

        self.lst = [0, 1, 2, 3]
        self.nums = rand.sample(self.lst, 4)
        self.numOne = self.nums[0]
        self.numTwo = self.nums[1]

    def test_QA_constructor(self):
        ##Assert conversion from dataframe to array works.
        self.assertEqual(type(methods.QA_constructor(self.QADataFrame, "Computer Science")), list)
        self.assertNotEqual(type(methods.QA_constructor(self.QADataFrame, "Computer Science")), type(self.QADataFrame))

        #Assert array contains QA types
        self.assertEqual(type(methods.QA_constructor(self.QADataFrame, "Computer Science")[1]), quiz_main_testcopy.QA)

        #Assert that the questions and answers from the dataframe input are correctly converted
        self.assertEqual(methods.QA_constructor(self.QADataFrame, "Computer Science")[1].question, "Software")
        self.assertEqual(methods.QA_constructor(self.QADataFrame, "Computer Science")[1].answer, "Set of instructions that tells the hardware what to do. It is what guides the hardware and tells it how to accomplish each task.")

        #Assert that two (random) different answers in the QA array are not duplicates (i.e. same as each other)
        self.assertNotEqual(methods.QA_constructor(self.QADataFrame, "ComputerScience")[self.numOne].answer, methods.QA_constructor(self.QADataFrame, "Computer Science")[self.numTwo].answer)
        self.assertNotEqual(methods.QA_constructor(self.QADataFrame, "ComputerScience")[self.numOne].question, methods.QA_constructor(self.QADataFrame, "Computer Science")[self.numTwo].question)


    def test_hints(self):
        #Assert that output for hints is different than input, where input is random answers from the answers column in dataframe
        self.assertNotEqual(methods.hints(self.QADataFrame.iloc[self.numOne, 1]), methods.hints(self.QADataFrame.iloc[self.numOne, 1]))

        #Assert that output for hints for sentences is different from input for sentences
        self.assertNotEqual(methods.hints(self.QADataFrame.iloc[2, 1]), methods.hints(self.QADataFrame.iloc[2, 1]))

        #Assert that output for hints for single words is different from input for single words
        self.assertNotEqual(methods.hints(self.QADataFrame.iloc[3, 1]), methods.hints(self.QADataFrame.iloc[3, 1]))

        #Assert that hint will print out at least one underscore
        self.assertGreater(str.count(methods.hints(self.QADataFrame.iloc[self.numOne, 1]), "_"), 0)



    def tearDown(self):
        #Delete random generated numbers after each function call and generate new ones
        self.lst, self.nums, self.numOne, self.numTwo = None, None, None, None

    @classmethod
    def tearDownClass(self):
        #Delete question/answer dataframe after testing is done
        self.data, self.QADataFrame = None, None
    unittest.main(argv=[''], verbosity = 2, exit=False)



def url_quizlet(topic):
	assert type(topic) == str, "Error: topic must be a string."
	allowedchars = 'abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ'
	assert set(topic).issubset(allowedchars), "Please enter only letters."
	topicF = topic.replace(" ", "-")
	url = "https://quizlet.com/search?query=" + topicF + "&type=sets&page=1&creator=teacher" #&useOriginal=
	return url


def url_cram(topic):
	assert type(topic) == str, "Error: topic must be a string."
	allowedchars = 'abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ'
	assert set(topic).issubset(allowedchars), "Please enter only letters."
	topicF = topic.replace(" ", "+")
	url = "https://www.cram.com/search?query=" + topicF + "&search_in%5B%5D=title&search_in%5B%5D=body&search_in%5B%5D=subject&search_in%5B%5D=username&image_filter=exclude_imgs&period=any"
	return url



def webscrape_quizlet(url=None, setNum=1, override=None):
    from selenium import webdriver
    from selenium.webdriver.support.ui import WebDriverWait
    from selenium.webdriver.common.by import By
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.webdriver.common.keys import Keys
    import time

    if (override==None):
	    try:
	        import translation
	    except:
	        from . import translation
	    translation.speed_warning()

	    ###search page
	    driver = webdriver.Firefox()
	    driver.get(url)

	    try:
	        cardset = WebDriverWait(driver, 15).until(EC.visibility_of_all_elements_located(('xpath', "//*[@class='SearchResultsPage-result' and @data-page-depth='" + str(setNum) + "']//*[@class='AssemblyLink AssemblyLink--medium AssemblyLink--title']")))

	        cardseturl = None
	        cardseturl = cardset[0].get_attribute('href')

	        driver.quit()
	    except NoSuchElementException:
	        driver.quit()
	        print("No quizzes found for user topic.")
	        return

	    except:
	        driver.quit()
	        print("Request timed out.")
	        return
    else:
        cardseturl = override


    ###card set page
    driver = webdriver.Firefox()
    driver.get(cardseturl)

    time.sleep(2)
    html = driver.find_element('tag name', 'html')
    html.send_keys(Keys.PAGE_DOWN)

    questionlist = WebDriverWait(driver, 45).until(EC.visibility_of_all_elements_located(('xpath', "//*[@class='SetPageTerms-term' and @aria-label='Term']//*[@class='SetPageTerm-side SetPageTerm-smallSide']//*[@class='TermText notranslate lang-en']")))

    answerlist = WebDriverWait(driver, 45).until(EC.visibility_of_all_elements_located(('xpath', "//*[@class='SetPageTerms-term' and @aria-label='Term']//*[@class='SetPageTerm-side SetPageTerm-largeSide']//*[@class='TermText notranslate lang-en']")))

    QAdataframe = dataframe_builder(questionlist, answerlist)

    driver.quit()

    print("Done.")

    return QAdataframe

    #can delete later
    #attrs = driver.execute_script('var items = {}; for (index = 0; index < arguments[0].attributes.length; ++index) { items[arguments[0].attributes[index].name] = arguments[0].attributes[index].value }; return items;', cardset[0])
    #print(attrs)







def webscrape_cram(url=None, setNum=1, override=None):
	from selenium import webdriver
	from selenium.webdriver.support.ui import WebDriverWait
	from selenium.webdriver.common.by import By
	from selenium.webdriver.support import expected_conditions as EC
	from selenium.webdriver.common.keys import Keys
	import time
	import pandas as pd


	if (override==None):
		try:
			import translation
		except:
			from . import translation
			translation.speed_warning()

		###search page
		driver = webdriver.Firefox()
		driver.get(url)


		try:
			cardset = WebDriverWait(driver, 45).until(EC.visibility_of_all_elements_located(('xpath', "//*[@class='searchResults']//*[@class='info']/h3/a")))

			cardseturl = cardset[setNum-1].get_attribute('href')

			driver.quit()

		except NoSuchElementException:
			driver.quit()
			print("No quizzes found for user topic.")
			return

		except:
			driver.quit()
			print("Request timed out.")
			return
	else:
		cardseturl = override


	###card set page
	driver = webdriver.Firefox()
	driver.get(cardseturl)

	time.sleep(2)
	html = driver.find_element('tag name', 'html')
	html.send_keys(Keys.PAGE_DOWN)

	from selenium.webdriver.support.ui import Select
	select = Select(driver.find_element('id', 'tablePagination_rowsPerPage'))
	select.select_by_visible_text('100')

	questionlist = driver.find_elements('xpath', "//*[@class='flashCardsListingTable']//*[@class='front_text card_text']")

	answerlist = driver.find_elements('xpath', "//*[@class='flashCardsListingTable']//*[@class='back_text card_text']")

	#questionlist = WebDriverWait(driver, 60).until(EC.visibility_of_all_elements_located(('xpath', "//*[@class='flashCardsListingTable']//*[@class='front_text card_text']")))

	#answerlist = WebDriverWait(driver, 60).until(EC.visibility_of_all_elements_located(('xpath', "//*[@class='flashCardsListingTable']//*[@class='back_text card_text']")))

	QAdataframe = dataframe_builder(questionlist, answerlist)

	driver.quit()

	print("Done.")

	return QAdataframe





def dataframe_builder(col1, col2):
    import pandas as pd

    questiontext = []
    answertext = []


    for i in range(len(col1)):
        if col1[i].text:
            questiontext.append(col1[i].text)
        else:
            questiontext.append("<image>")

    for i in range(len(col2)):
        if col2[i].text:
            answertext.append(col2[i].text)
        else:
            answertext.append("<image>")

    #creating dataframe for export
    questions = []
    answers = []


    QAdataframe = pd.DataFrame({'questions': questiontext, 'answers': answertext})
    return QAdataframe



import sys
import os
import re
packagepath = os.path.abspath(__file__)
packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
if not packagepath in sys.path:
	sys.path.append(packagepath)

from core import quiz_methods_testcopy as methods
from core import quiz_main_testcopy
from core import quiz_main_testcopy as main

import unittest
import pandas as pd
import random as rand

class TestMain(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        #Generate a bank of questions/topics to assign to quiz objects
        self.wordlst = ["Question/Topic One", "Question/Topic Two", "Question/Topic Three", "Question/Topic Four",
                        "Question/Topic Five"]
    def setUp(self):
        #Generate random integers (sample without replacement) to randomly pick two different questions or topics
        #to assign to quiz objects for testing.
        self.lst = [0, 1, 2, 3]
        self.nums = rand.sample(self.lst, 4)
        self.numOne = self.nums[0]
        self.numTwo = self.nums[1]
        self.numThree = self.nums[2]
        self.numFour = self.nums[3]



    def test_Quiz(self):
        quizObj = main.Quiz()
        #Assert instantiating quiz object creates quiz type instance object
        self.assertEqual(type(quizObj), quiz_main_testcopy.Quiz)

        #Assert that instantiating quiz object contains attributes of topic, languageDict, etc...
        self.assertEqual(" ".join(vars(quizObj).keys()), "topic languageDict site setNum language results")

        #Assert that instantiating quiz object will have default attribute values of biology, etc...
        self.assertEqual(str(vars(quizObj).values()), "dict_values(['Biology', {'english': 'en', 'chinese': 'zh-tw', 'french': 'fr'}, 'quizlet', 1, 'en', 'on'])")

        #Assert that constructors with arguments will modify quiz instance attributes (assigning random topics/questions)
        quizObjOne = main.Quiz(self.wordlst[self.numOne])
        quizObjTwo = main.Quiz(self.wordlst[self.numTwo])

        #Two quiz objects with two different random questions aren't equal to each other
        self.assertNotEqual(quizObjOne.topic, quizObjTwo.topic)

        #Assigning value in constructor argument changes default attribute to one specified in argument
        self.assertNotEqual(quizObjOne.topic, "Biology")
        self.assertEqual(quizObjOne.topic, self.wordlst[self.numOne])
    def test_QA(self):
        qaObjOne = main.QA(self.wordlst[self.numOne], self.wordlst[self.numTwo], "Topic")
        qaObjTwo = main.QA(self.wordlst[self.numThree], self.wordlst[self.numFour], "Topic")

        #Assert QA object instantiates correctly
        self.assertEqual(qaObjOne.question, self.wordlst[self.numOne])
        self.assertEqual(type(qaObjOne), quiz_main_testcopy.QA)
        #Assert instantiates with attributes of question, etc.
        self.assertEqual(" ".join(vars(qaObjOne).keys()), "question answer topic mark")

        #Assert different QA objects containing two different random questions/topics are different to each other
        self.assertNotEqual(qaObjOne.question, qaObjTwo.question)

        #Assert that we can modify QA questions/answers by simple assignment statements
        qaObjOne.question = "Modified first question"

        self.assertEqual(qaObjOne.question, "Modified first question")
        self.assertNotEqual(qaObjOne.question, self.wordlst[self.numOne])
    def tearDown(self):
        #Delete random generated numbers after each function call
        self.lst, self.nums, self.numOne, self.numTwo, self.numThree, self.numFour = None, None, None, None, None, None
    @classmethod
    def tearDownClass(self):
        #Delete bank of questions and topics after testing is done
        self.wordlst = None

    unittest.main(argv=[''], verbosity = 2, exit=False)



def QA_constructor(QAdataframe, topic):
	import pandas as pd
	from . import quiz_main_testcopy
	array = []
	for i in range(len(QAdataframe.index)):
		temp = quiz_main_testcopy.QA(QAdataframe.loc[i, "questions"], QAdataframe.loc[i, "answers"], topic)
		array.append(temp)

	return array





def take_the_quiz(quiz):
	from py_quizlet_cram.core import quiz_main_testcopy
	import time
	from IPython.display import clear_output
	assert isinstance(quiz, quiz_main_testcopy.Quiz)

	length = quiz.quiz_length
	array = quiz.QAs

	print_options(quiz)


	theInput = None
	for i in range(quiz.quiz_length):
		print("Question " + str(i + 1) + " of " + str(quiz.quiz_length) + ": " + quiz.QAs[i].question +
		"\nPress enter for the answer, hint for hints, or exit to exit. \n")
		while theInput not in ["", "exit"]:
			theInput = input()
			if theInput == "hint":
				print(hints(quiz.QAs[i].answer))
		if theInput == "":
			print("Answer:")
			print(quiz.QAs[i].answer)
			if quiz.results == "on":
				rightwrong = input("Did you understand this concept? (Y/N). \n").lower()
				while not ((rightwrong == "y") or (rightwrong == "n")):
					rightwrong = input("Please specify Y or N. \n").lower()
				array[i].mark = rightwrong
				if rightwrong == "y" or rightwrong == "n":
					clear_output(wait=False)
					theInput = "placeHolder"

			else:
				theInput = input()
				if theInput != "exit":
					clear_output(wait=False)
					theInput = "placeHolder"
				elif theInput == "exit":
					break
		elif theInput == "exit":
			break

	print("\nWell done! Quiz is finished. \n")

	#call plotting methods
	if quiz.results.lower() == "on":
		results_plot(quiz)




def print_options(quiz):
	import time
	print("You are about to start the quiz with the following options...:")
	time.sleep(2)
	print("Your topic:", quiz.topic)
	time.sleep(1)
	print("With questions from:", quiz.site)
	time.sleep(1)
	print("Using question bank number:", quiz.setNum)
	time.sleep(1)
#	print("Total number of questions:", quiz.quiz_length)
	time.sleep(1)
	print("Language:", quiz.language)
	time.sleep(1)
	print("With results:", quiz.results)
	time.sleep(1)
	print("Good luck!")



def results_plot(quiz):
	import matplotlib.pyplot as plt

	array = []
	length = quiz.quiz_length
	for i in range(length):
		array.append(quiz.QAs[i].mark)

	if (quiz.results == "off"):
		print("Cannot print results if results was set to \'off\'. \n")
		return
	if array.count('y') + array.count('n') == 0:
		pass
	else:

		plotting = [array.count('y'), array.count('n')]

		print("Right:", plotting[0])
		print("Wrong:", plotting[1])
		print("Grade:", round(plotting[0]/length * 100), "%")

		plt.pie(plotting, labels = ['Right', 'Wrong'])
		plt.show()





def hints(word):
	import random
	from IPython.display import clear_output
	hintAnswer = []
	if len(word.split(" ")) != 1:
		randomLister = []
		splitSentence = word.split(" ")
		for i in range(len(splitSentence)):
			randHint = random.randint(0, len(splitSentence))
			randomLister.append(randHint)
		for j in range(len(splitSentence)):
			appendWord = splitSentence[j]
			if j in randomLister:
				hintAnswer.append(appendWord)
			if j not in randomLister:
				theWord = splitSentence[j]
				for k in range(len(theWord)):
					theWord = theWord.replace(theWord[k], "_")
				hintAnswer.append(theWord)
	elif len(word.split(" ")) == 1:
		randList = []
		theWord = word
		for l in range(len(theWord)):
			randHint = random.randint(0, len(theWord))
			randList.append(randHint)
		for m in range(len(theWord)):
			if m not in randList:
				theWord = theWord.replace(theWord[m], "_")
		hintAnswer.append(theWord)
	return " ".join(hintAnswer)



########
#NOTE FOR TA:
#there is only one set case here, as there is only testable function in thie module. To compensate for this, I added an extra test case to 'webscraping_testing.py'


import unittest


#translate
class translation(unittest.TestCase): # test class
    import pandas as pd

    #trying to import package
    import sys
    import os
    import re
    packagepath = os.path.abspath(__file__)
    packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
    if not packagepath in sys.path:
        sys.path.append(packagepath)

    from network import translation



    #set up and tear down
    def setUp(self):
        import pandas as pd
        #print("setup")
        self.en = {'questions':['pencil', 'snake'], 'answers':['hammer', 'nail']}
        self.fr = {'questions':['crayon', 'serpent'], 'answers':['marteau', 'clou']}
        self.ch = {'questions':['蠟筆', '蛇'], 'answers':['錘子', '指甲']}
        self.df_en = pd.DataFrame(self.en)
        self.df_fr = pd.DataFrame(self.fr)
        self.df_ch = pd.DataFrame(self.ch)

    def tearDown(self):
        #print("teardown")
        del self.en
        del self.fr
        del self.ch
        del self.df_en
        del self.df_fr
        del self.df_ch

    #translate function
    def test_translate(self): # test case
        self.assertEqual(None, self.pd.testing.assert_frame_equal(self.translation.translate(self.df_en, 'en'), self.df_en))
        self.assertEqual(None, self.pd.testing.assert_frame_equal(self.translation.translate(self.df_en, 'fr'), self.df_fr))
        self.assertEqual(None, self.pd.testing.assert_frame_equal(self.translation.translate(self.df_en, 'zh-tw'), self.df_ch))
unittest.main(argv=[''], verbosity=2, exit=False)







#speed_warning
class testing_speed_warning(unittest.TestCase): # test class

    #trying to import package
    import sys
    import os
    import re
    packagepath = os.path.abspath(__file__)
    packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
    if not packagepath in sys.path:
        sys.path.append(packagepath)

    from network import translation


    #set up and tear down
    def setUp(self):
        import io
        import sys
        self.capturedOutput = io.StringIO()
        sys.stdout = self.capturedOutput ###
        self.translation.speed_warning()
        sys.stdout = sys.__stdout__
        self.test_output = self.capturedOutput.getvalue()
        self.expected_output = 'Functions requiring internet connection may take a while. Please do not interact with your device until the process is finished.\nStarting in:\n3\n2\n1\n'

    def tearDown(self):
        #print("teardown")
        del self.capturedOutput
        del self.test_output
        del self.expected_output

    #test_speed_warning function
    def test_speed_warning(self): # test case
        self.assertEqual(self.test_output, self.expected_output)
unittest.main(argv=[''], verbosity=2, exit=False)



class Quiz():
	#importing stuff
	import sys
	import os
	import re
	packagepath = os.path.abspath(__file__)
	packagepath = re.sub(r'\/[^\/]*\/{1}[^\/]*(\.).*', '', packagepath)
	if not packagepath in sys.path:
		sys.path.append(packagepath)
	try:
		from py_quizlet_cram.network import webscraping
		from py_quizlet_cram.network import translation
	except:
		from network import webscraping
		from network import translation

	from . import quiz_methods

	#checking internet connection
	if translation.internet_checker():
		print("Internet connection confirmed.")
	else:
		raise Exception("Internet connection is required!")

	def __init__(self, topic="", site = "Quizlet", setNum = 1, language = "english", results = "on"):
		self.topic = input("\nPlease enter a topic that you want to be quizzed on: ")
		self.languageDict = {"english": "en", "chinese": "zh-tw", "french": "fr"}
		self.site = site.lower()
		self.setNum = setNum #up to a max of 8
		self.language = self.languageDict[language]
		self.results = results.lower()

		#user_control
		user_control(self)






def quiz_create(theQuiz):
	if theQuiz.site == "quizlet":
		theQuiz.url = theQuiz.webscraping.url_quizlet(theQuiz.topic)
		try:
			theQuiz.__array = theQuiz.webscraping.webscrape_quizlet(theQuiz.url, theQuiz.setNum)
		except:
			print("Sorry, no quizzes were found for: ", theQuiz.topic)
	elif theQuiz.site == "cram":
		theQuiz.url = theQuiz.webscraping.url_cram(theQuiz.topic)
		try:
			theQuiz.__array = theQuiz.webscraping.webscrape_cram(theQuiz.url, theQuiz.setNum)
		except:
			print("Sorry, no quizzes were found for: ", theQuiz.topic)
	else:
		raise Exception("Not a valid site name. Please enter either \'Quizlet\' or \'Cram\'. Default is \'Quizlet\'.")

	#translate
	if theQuiz.language != 'en':
		from py_quizlet_cram.network import translation
		translation.translate(theQuiz.__array, theQuiz.language)

	theQuiz.quiz_length = len(theQuiz.__array.index)
	theQuiz.QAs = theQuiz.quiz_methods.QA_constructor(theQuiz.__array, theQuiz.topic)







class QA():
	def __init__(self, question, answer, topic):
		self.question = question
		self.answer = answer
		self.topic = topic
		self.mark = None

	def set_mark(value):
	#0 for wrong, 1 for correct
		self.mark = value






def user_control(theQuiz):
	import py_quizlet_cram.core.quiz_methods as quiz_methods
	from IPython.display import clear_output
	theInput = ""

	#main loop
	while theInput.lower() != "exit":
		clear_output(wait=True)
		theInput = input("You are about to start a quiz on " + theQuiz.topic + " from the site " + theQuiz.site
	                   + ".\nPlease choose from the following options: \n   Start (starts the quiz!)"
	                   + "\n   Settings (change settings)\n   Exit (quit)\n")
		if theInput == "start":
	        	quiz_create(theQuiz)
	        	quiz_methods.take_the_quiz(theQuiz)
	        	theInput = "exit"
		if theInput.lower() == "settings":
			while theInput.lower() != "exit":
				clear_output(wait=True)
				theInput = input("What options do you want to change? \n   Topic (Quiz topic on "
	                             + theQuiz.topic + ")\n   Language (" + theQuiz.language + ")\n   Site (" + theQuiz.site
	                             + ")\n   Set (question bank number " + str(theQuiz.setNum) + ")\n   Results (Display results " + theQuiz.results + ")\n   Back\n   Exit\n")
				if theInput.lower() == "back":
					break;
				elif theInput.lower() == "topic":
					theInput = input("Enter topic you want to be quizzed on: \n")
					theQuiz.topic = theInput
				elif theInput.lower() == "language":
					theInput = input("Language options are: " + " ".join(['{0}'.format(k, v) for k, v in theQuiz.languageDict.items()]).title() + "\n")
					while theInput not in ["chinese", "english", "french"]:
						theInput = input("Please enter: " + " ".join(['{0}'.format(k, v) for k, v in theQuiz.languageDict.items()]).title() + "\n")
					if theInput.lower() == "english":
						theQuiz.language = theQuiz.languageDict[theInput]
					elif theInput.lower() == "chinese":
						theQuiz.language = theQuiz.languageDict[theInput]
					elif theInput.lower() == "french":
						theQuiz.language = theQuiz.languageDict[theInput]
				elif theInput.lower() == "results":
					theInput = input("Please specify \'on\' to display results, or \'off\' to not display results.\n")
					while theInput not in ["on", "off"]:
						theInput = input("Please enter either \'on\' or \'off\'.\n")
					theQuiz.results = theInput
				elif theInput.lower() == "site":
					theInput = input("Enter the website you want questions from (Quizlet or Cram):\n")
					while theInput not in ["quizlet", "cram"]:
						theInput = input("Please enter either Quizlet or Cram.\n")
					if theInput.lower() == "quizlet":
						theQuiz.site = "quizlet"
					elif theInput.lower() == "cram":
						theQuiz.site = "cram"
				elif theInput.lower() == "set":
					theInput = input("Please choose a question set number from 1 to 8.\n")
					while str(theInput) not in ["1", "2", "3", "4", "5", "6", "7", "8"]:
						theInput = input("Please only specify question bank values from 1 to 8.\n")
					theQuiz.setNum = int(theInput)



from setuptools import setup, find_packages

setup(
    name='py_quizlet_cram_stepthree',
    version='0.1',
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    description='A package for taking quizlet and cram quizzes',
	url='https://github.com/jonahedmundson/py_quizlet_cram_stepthree/tree/main',
    author='Jonah Edmunson and Matthew Yau',
    author_email='jonahedmundson@gmail.com'
)
